namespace AMDiS {

  template <typename T>
  void Operator::addSecondOrderTerm(T *term)
  {
    secondOrder[0].push_back(term);
    term->operat = this;
    auxFESpaces.insert(auxFESpaces.end(), 
		       term->getAuxFESpaces().begin(),
		       term->getAuxFESpaces().end());

    for (int i = 1; i < omp_get_overall_max_threads(); i++) {
      T *newTerm = new T(static_cast<const T>(*term));
      secondOrder[i].push_back(newTerm);
    }
  }

  template <typename T>
  void Operator::addFirstOrderTerm(T *term,
				   FirstOrderType type)
  {
    if (type == GRD_PSI) {
      firstOrderGrdPsi[0].push_back(term);
    } else {
      firstOrderGrdPhi[0].push_back(term);
    }
    term->operat = this;
    auxFESpaces.insert(auxFESpaces.end(), 
		       term->getAuxFESpaces().begin(),
		       term->getAuxFESpaces().end());

    for (int i = 1; i < omp_get_overall_max_threads(); i++) {
      T *newTerm = new T(static_cast<const T>(*term));
      if (type == GRD_PSI) {
	firstOrderGrdPsi[i].push_back(newTerm);
      } else {
	firstOrderGrdPhi[i].push_back(newTerm);
      } 
    }
  }

  template <typename T>
  void Operator::addZeroOrderTerm(T *term)
  {
    zeroOrder[0].push_back(term);
    term->operat = this;
    auxFESpaces.insert(auxFESpaces.end(), 
		       term->getAuxFESpaces().begin(),
		       term->getAuxFESpaces().end());

    for (int i = 1; i < omp_get_overall_max_threads(); i++) {
      T *newTerm = new T(static_cast<const T>(*term));
      zeroOrder[i].push_back(newTerm);
    }
  }


}
