// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file PeriodicMap.h */

#ifndef AMDIS_PERIODICMAP_H
#define AMDIS_PERIODICMAP_H

#include "Global.h"

#include <map>

namespace AMDiS {

  class PeriodicMap
  {
  public:
    void setEntry(DegreeOfFreedom key, DegreeOfFreedom entry) {
      // no trivial entries!
      if (key == entry) 
	return;

      // if a key equal to entry exists ...
      if (getEntry(entry) >= 0) {
	if (getEntry(entry) == key) {
	  return;
	}
	// ... let it be equal entries
	setEntry(key, getEntry(entry));
	return;
      }

      // replace entries equal to key
      std::map<DegreeOfFreedom, DegreeOfFreedom>::iterator it;
      for (it = periodicMap.begin(); it != periodicMap.end(); ++it) {
	if (it->second == key) {
	  it->second = entry;
	}
      }
    
      // if key exists already
      if (getEntry(key) >= 0) {
	// insert new entry with old entry as key
	setEntry(getEntry(key), entry);
      }

      // set entry
      periodicMap[key] = entry;
    };

    DegreeOfFreedom getEntry(DegreeOfFreedom key) {
      std::map<DegreeOfFreedom, DegreeOfFreedom>::iterator it;
      it = periodicMap.find(key);
      if (it != periodicMap.end()) {
	return it->second;
      } else {
	return -1;
      }
    };

  

  protected:
    std::map<DegreeOfFreedom, DegreeOfFreedom> periodicMap;
  };

}

#endif
