#include "StandardProblemIteration.h"
#include "AdaptInfo.h"
#include "ProblemStatBase.h"
#include "Global.h"
#include "OpenMP.h"

namespace AMDiS {

  int StandardProblemIteration::info = 10;

  void StandardProblemIteration::beginIteration(AdaptInfo *adaptInfo) 
  {
    FUNCNAME("StandardProblemIteration::beginIteration()");

    INFO(info, 4)("\n");
    INFO(info, 4)("begin of iteration number: %d\n", adaptInfo->getSpaceIteration() + 1);
    INFO(info, 4)("=============================\n");
  }

  Flag StandardProblemIteration::oneIteration(AdaptInfo *adaptInfo, Flag toDo) 
  {
    FUNCNAME("StandardProblemIteration::oneIteration()");

    Flag flag = buildAndAdapt(adaptInfo, toDo);

    if (toDo.isSet(SOLVE))
      problem->solve(adaptInfo, false);

    if (toDo.isSet(SOLVE_RHS))
      problem->solve(adaptInfo, true);

    if (toDo.isSet(ESTIMATE)) 
      problem->estimate(adaptInfo);

    return flag;
  }

  void StandardProblemIteration::endIteration(AdaptInfo *adaptInfo) 
  {
    FUNCNAME("StandardProblemIteration::endIteration()");

    INFO(info, 4)("\n");
    INFO(info, 4)("end of iteration number: %d\n", adaptInfo->getSpaceIteration() + 1);
    INFO(info, 4)("=============================\n");
  }

  Flag StandardProblemIteration::buildAndAdapt(AdaptInfo *adaptInfo, Flag toDo)
  {
    FUNCNAME("StandardProblemIteration::buildAndAdapt()");

    Flag flag = 0, markFlag = 0;

    if (toDo.isSet(MARK))
      markFlag = problem->markElements(adaptInfo);
    else
      markFlag = 3;

    if (toDo.isSet(BUILD))
      problem->buildBeforeRefine(adaptInfo, markFlag);

    // refine
    if (toDo.isSet(ADAPT) && markFlag.isSet(MESH_REFINED))
      flag = problem->refineMesh(adaptInfo);

    if (toDo.isSet(BUILD))
      problem->buildBeforeCoarsen(adaptInfo, markFlag);

    // coarsen
    if (toDo.isSet(ADAPT) && markFlag.isSet(MESH_COARSENED))
      flag |= problem->coarsenMesh(adaptInfo);

    if (toDo.isSet(BUILD))
      problem->buildAfterCoarsen(adaptInfo, markFlag, true, true);

    if (toDo.isSet(BUILD_RHS))
      problem->buildAfterCoarsen(adaptInfo, markFlag, false, true);

    return flag;
  }

  std::string StandardProblemIteration::getName()
  {
    return problem->getName();
  }

  void StandardProblemIteration::serialize(std::ostream &out)
  {
    problem->serialize(out);
  }

  void StandardProblemIteration::deserialize(std::istream &in)
  {
    problem->deserialize(in);
  }
};
