// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file StandardProblemIteration.h */

#ifndef AMDIS_STANDARDPROBLEMITERATION_H
#define AMDIS_STANDARDPROBLEMITERATION_H

#include "AMDiS_fwd.h"
#include "ProblemIterationInterface.h"

namespace AMDiS {

  /// A master problem for a single non coupled problem.
  class StandardProblemIteration : public ProblemIterationInterface
  {
  public:
    /// constructor
    StandardProblemIteration(ProblemStatBase *prob)
      : problem(prob)
    {}

    virtual ~StandardProblemIteration() {}

    /// Implementation of \ref ProblemIterationIterface::beginIteration()
    virtual void beginIteration(AdaptInfo *adaptInfo);

    /// Implementation of \ref ProblemIterationInterface::oneIteration()
    virtual Flag oneIteration(AdaptInfo *adaptInfo, Flag toDo);

    /// Implementation of \ref ProblemIterationInterface::endIteration()
    virtual void endIteration(AdaptInfo *adaptInfo);

    /// Implementation of \ref ProblemIterationInterface::getNumProblems()
    int getNumProblems() 
    { 
      return 1; 
    }

    /// Implementation of \ref ProblemIterationInterface::getProblem(int)
    ProblemStatBase *getProblem(int number = 0) 
    {
      return problem;
    }

    /// Returns the name of the problem.
    virtual std::string getName();

    /// Function that serializes the problem plus information about the iteration.
    virtual void serialize(std::ostream &out);

    /// Function that deserializes the problem plus information about the iteration.
    virtual void deserialize(std::istream &in);

    /// Nested assemblage and mesh adaption.
    Flag buildAndAdapt(AdaptInfo *adaptInfo, Flag toDo);

  protected:
    /// The problem to solve.
    ProblemStatBase *problem;

    /// Info level
    static int info;
  };

}

#endif

