#include <algorithm>

#include "SurfaceQuadrature.h"
#include "Quadrature.h"

namespace AMDiS {

  SurfaceQuadrature::SurfaceQuadrature(Quadrature *quad,
				       VectorOfFixVecs<DimVec<double> > &coords)
    : Quadrature((quad->getName() + " surface").c_str(),
		 quad->getDegree(),
		 quad->getDim() + 1,
		 quad->getNumPoints(),
		 NULL,
		 quad->getWeight()), 
      quad_(quad),
      coords_(coords)
  {
    lambda = new VectorOfFixVecs<DimVec<double> >(dim, n_points, NO_INIT);

    // for each integration point
    for (int i = 0; i < n_points; i++) {
      // get coords of quadrature point in dim-1
      DimVec<double> origin = quad->getLambda(i);

      for (int j = 0; j < dim + 1; j++)
	(*lambda)[i][j] = 0.0;

      for (int j = 0; j < dim; j++)
	for (int k = 0; k < dim + 1; k++)
	  (*lambda)[i][k] += origin[j] * coords_[j][k];
    }
  }

  void SurfaceQuadrature::scaleSurfaceQuadrature(VectorOfFixVecs<DimVec<double> >&coords)
  {
    // copy coords
    for (int i = 0; i < dim; i++)
      coords_[i] = coords[i];
  
    // for each integration point
    for (int i = 0; i < n_points; i++) {
      // get coords of quadrature point in dim-1
      DimVec<double> origin = quad_->getLambda(i);

      for (int j = 0; j < dim + 1; j++)
	(*lambda)[i][j] = 0.0;

      for (int j = 0; j < dim; j++)
	for (int k = 0; k < dim+1; k++)
	  (*lambda)[i][k] += origin[j] * coords_[j][k];
    }
  }

}
