// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file SurfaceQuadrature.h */

#ifndef AMDIS_SURFACEQAUDRATURE_H
#define AMDIS_SURFACEQAUDRATURE_H

#include <list>
#include "FixVec.h"
#include "Quadrature.h"

namespace AMDiS {

  /** 
   * \ingroup Integration
   *
   * \brief
   * Quadrature for element surfaces. Uses the functionality of the standard
   * Quadrature class for dim-1 but calculates new barycentric coordinates
   * for the element sides.
   */
  class SurfaceQuadrature : public Quadrature
  {
  public:
    /// Constructs a SurfaceQuadrature based on a standard Quadrature of dim-1.
    SurfaceQuadrature(Quadrature *quad, VectorOfFixVecs<DimVec<double> >& coords);

    /// Destructor.
    ~SurfaceQuadrature()
    {}

    /// Adapts SurfaceQuadrature to \ref coords.
    void scaleSurfaceQuadrature(VectorOfFixVecs<DimVec<double> > &coords);

  protected:
    /// Pointer to the original quadrature
    Quadrature *quad_;

    VectorOfFixVecs<DimVec<double> > coords_;
  };

}

#endif // AMDIS_SURFACEQAUDRATURE_H
