// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file TecPlotWriter.h */

#ifndef AMDIS_TEC_PLOT_WRITER_H
#define AMDIS_TEC_PLOT_WRITER_H

#include <fstream>
#include <vector>
#include "FixVec.h"
#include "AMDiS_fwd.h"

namespace AMDiS {

  /** \ingroup Output
   * \brief
   * Writes a DOFVector in TecPlot format to a ascii file
   */
  template<typename T>
  class TecPlotWriter
  {
  public:
    /** \brief
     * Writes the DOFVector values to the file with given name. Different 
     * DOFVectors belonging to the same triangulation can be written to one
     * file by setting additional=true for all DOFVectors except for the first 
     * one.
     */
    static void writeValues(T* values,
			    const char* filename,
			    const char* plotTitle,
			    bool additional=false);

  private:
    static int writeValuesFct(ElInfo* elinfo);
    static int writeIndicesFct(ElInfo* elinfo);
    static void writeVarName(int dim);

  private:
    class VertexInfo {
    public:
      WorldVector<double> coord;

      int vertex_index;

    public:
      inline bool operator==(const WorldVector<double>& coor) 
      {
	return (coor == coord);
      }

      inline bool operator!=(const WorldVector<double>& coor) 
      {
	return !(*this == coor);
      }
    };

    typedef std::vector<VertexInfo> DOFCoords;

    static T* values;
    static int nv;
    static std::ofstream* outFile;
    static bool writeCoords;
    static std::vector<DOFCoords> dofCoords;
  };

}

#include "TecPlotWriter.hh"

#endif
